/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos
 * Encryption/decryption library. This library is free software;
 * you can redistribute it and/or modify it under the terms of the
 * GNU Library General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

/* $Id: mcrypt.h,v 1.6 2000/03/01 09:48:37 nikos Exp $ */

#define MCRYPT_API_VERSION 20000222

#ifdef __cplusplus
extern "C" {
#endif

/* Definitions
 */
#define MCRYPT_UNKNOWN_ERROR -1
#define MCRYPT_ALGORITHM_MODE_INCOMPATIBILITY -2
#define MCRYPT_KEY_LEN_ERROR -3
#define MCRYPT_MEMORY_ALLOCATION_ERROR -4
#define MCRYPT_UNKNOWN_MODE -5
#define MCRYPT_UNKNOWN_ALGORITHM -6



/* generic - high level functions. 
 */
	    int mcrypt_module_open(char *algorithm,
					   char *a_directory, char *mode,
				   char *m_directory);
	    int mcrypt_module_close( int td);


/* returns thread descriptor */

	int mcrypt_generic_init(const int td, void *key, int lenofkey,
				void *IV);
	int mcrypt_generic_end(const int td);
	int mdecrypt_generic(int td, void *plaintext, int len);
	int mcrypt_generic(int td, void *plaintext, int len);

/* extra functions */

	int (mcrypt_self_test) (int);
	int (mcrypt_get_block_size) (int);
	int (mcrypt_get_iv_size) (int);
	int (mcrypt_get_key_size) (int);

	int (is_ok_algorithm) (int);	/* if this algorithm exists returns 0 */
	int (is_ok_mode) (int);	/* if this mode exists returns 0 */

/* If this is a block algorithm returns 1 
 */
	int (is_block_algorithm) (int);	

/* If the mode operates in blocks returns 1 
 */
	int (is_block_mode) (int);

/* If the mode is for block algorithms it returns 1 
 */
	int (is_block_algorithm_mode) (int td);
	int mcrypt_mode_has_iv ( int td);

/* Return a string containing the name of the algorithm/mode
 */
	char *(mcrypt_get_algorithms_name) (int td);
	char *(mcrypt_get_modes_name) (int td);

char** mcrypt_list_algorithms ( char* libdir, int* size);
char** mcrypt_list_modes ( char* libdir, int *size);

 /* Frees the memory allocated by the mcrypt_list_xxx() functions. 
  */
void mcrypt_free_p(char** p, int size);

 /* If mcrypt_xxx functions return an error code, and you supply this
  * to this function, it will print a human readable message
  */
void mcrypt_perror(int err);

 /* Self test for the specified algorithm 
  */
int mcrypt_module_self_test (char* algorithm, char* libdir);

int mcrypt_is_block_algorithm(char *algorithm, char *a_directory);
int mcrypt_is_block_algorithm_mode(char *mode, char *m_directory);
int mcrypt_is_block_mode(char *mode, char *m_directory);

int mcrypt_get_algo_key_size(char *algorithm, char *a_directory);
int mcrypt_get_algo_block_size(char *algorithm, char *a_directory);

 /* Checks the version of the specified module 
  */
int mcrypt_algorithm_version(char *algorithm, char *a_directory);
int mcrypt_mode_version(char *mode, char *a_directory);

/* Compatibility definitions */
#define MCRYPT_BLOWFISH_448 MCRYPT_BLOWFISH
#define MCRYPT_BLOWFISH_256 MCRYPT_BLOWFISH
#define MCRYPT_BLOWFISH_192 MCRYPT_BLOWFISH
#define MCRYPT_BLOWFISH_128 MCRYPT_BLOWFISH
#define MCRYPT_RC2_1024 MCRYPT_RC2
#define MCRYPT_RC2_256 MCRYPT_RC2
#define MCRYPT_RC2_128 MCRYPT_RC2
#define MCRYPT_TWOFISH_256 MCRYPT_TWOFISH
#define MCRYPT_TWOFISH_192 MCRYPT_TWOFISH
#define MCRYPT_TWOFISH_128 MCRYPT_TWOFISH
#define MCRYPT_SERPENT_256 MCRYPT_SERPENT
#define MCRYPT_SERPENT_192 MCRYPT_SERPENT
#define MCRYPT_SERPENT_128 MCRYPT_SERPENT
#define MCRYPT_SAFER_64 MCRYPT_SAFER_SK64
#define MCRYPT_SAFER_128 MCRYPT_SAFER_SK128
#define MCRYPT_RIJNDAEL_256 MCRYPT_RIJNDAEL
#define MCRYPT_RIJNDAEL_192 MCRYPT_RIJNDAEL
#define MCRYPT_RIJNDAEL_128 MCRYPT_RIJNDAEL
#define MCRYPT_RIJNDAEL MCRYPT_RIJNDAEL_B128
#define MCRYPT_RC4 MCRYPT_ARCFOUR

#define MCRYPT_BLOWFISH 0
#define MCRYPT_DES 1 
#define MCRYPT_TRIPLEDES 2
#define MCRYPT_3DES TRIPLEDES
#define MCRYPT_THREEWAY 3
#define MCRYPT_3WAY MCRYPT_THREEWAY
#define MCRYPT_GOST 4
#define MCRYPT_SAFER_SK64 6
#define MCRYPT_SAFER_SK128 7
#define MCRYPT_CAST_128 8
#define MCRYPT_XTEA 9
#define MCRYPT_RC2 11
#define MCRYPT_TWOFISH 13
#define MCRYPT_CAST_256 17
#define MCRYPT_SAFERPLUS 18
#define MCRYPT_LOKI97 19
#define MCRYPT_SERPENT 22
#define MCRYPT_RIJNDAEL_B128 25
#define MCRYPT_RIJNDAEL_B192 26 /* B192 means blocksize 192 bits */
#define MCRYPT_RIJNDAEL_B256 27
#define MCRYPT_RIJNDAEL MCRYPT_RIJNDAEL_B128
#define MCRYPT_CRYPT 28
#define MCRYPT_WAKE 29
#define MCRYPT_ARCFOUR_IV 30
#define MCRYPT_RC6 100
#define MCRYPT_IDEA 101

#define MCRYPT_ARCFOUR 104
#define MCRYPT_PANAMA 105
#define MCRYPT_MARS 106

#define MCRYPT_CBC 0
#define MCRYPT_ECB 1
#define MCRYPT_CFB 2
#define MCRYPT_OFB 3
#define MCRYPT_nOFB 4
#define MCRYPT_STREAM 5

/* init function similar to 2.2 version:
 * it does not need to call mcrypt_module_open.
 */
	int mcrypt_generic_init_compat( const int mode, const int algorithm, 
			void *key, int lenofkey, void *IV);

#define mcrypt_generic_compat mcrypt_generic
#define mdecrypt_generic_compat mdecrypt_generic
#define mcrypt_generic_end_compat mcrypt_generic_end

int mcrypt_get_block_size_compat(int algorithm);
int mcrypt_get_key_size_compat(int algorithm);

int is_ok_algorithm_compat(int algorithm); /* if this algorithm exists returns 0 */
int is_ok_mode_compat(int mode); /* if this mode exists returns 0 */
int is_block_algorithm_compat(int algorithm); /* If this is a block algorithm returns 1 */
int is_block_mode_compat(int mode);

char* mcrypt_get_algorithms_name_compat(int algorithm);
char *mcrypt_get_modes_name_compat(int mode);


#ifdef __cplusplus
}
#endif
