/* Copyright (C) 1998,1999 Nikos Mavroyanopoulos This library is free
 * software; you can redistribute it and/or modify it under the terms of the
 * GNU Library General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any
 * later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#ifndef LIBDEFS_H
#define LIBDEFS_H
#include <libdefs.h>
#endif
#include <bzero.h>
#include <mcrypt.h>
#include <xmemory.h>
#include <mcrypt_internal.h>
#include <ctype.h>


void tolow( char* str, int size) {
int i;

	for(i=0;i<size;i++) {
		str[i]=tolower(str[i]);
	}
}



char* algo_num2char(int algo) {
	char* ret, *pointerTo_;


	switch (algo) {
		case MCRYPT_BLOWFISH:
			ret = strdup("blowfish");
			break;
		case MCRYPT_DES:
			ret = strdup("des");
			break;
		case MCRYPT_TRIPLEDES:
			ret = strdup("tripledes");
			break;
		case MCRYPT_THREEWAY:
			ret = strdup("threeway");
			break;
		case MCRYPT_GOST:
			ret = strdup("gost");
			break;
		case MCRYPT_SAFER_64:
			ret = strdup("safer-sk64");
			break;
		case MCRYPT_SAFER_128:
			ret = strdup("safer-sk128");
			break;
		case MCRYPT_CAST_128:
			ret = strdup("cast-128");
			break;
		case MCRYPT_XTEA:
			ret = strdup("xtea");
			break;
		case MCRYPT_RC2:
			ret = strdup("rc2");
			break;
		case MCRYPT_TWOFISH:
			ret = strdup("twofish");
			break;
		case MCRYPT_CAST_256:
			ret = strdup("cast-256");
			break;
		case MCRYPT_SAFERPLUS:
			ret = strdup("saferplus");
			break;
		case MCRYPT_LOKI97:
			ret = strdup("loki97");
			break;
		case MCRYPT_SERPENT:
			ret = strdup("serpent");
			break;
		case MCRYPT_RIJNDAEL_B128:
			ret = strdup("rijndael-128");
			break;
		case MCRYPT_RIJNDAEL_B192:
			ret = strdup("rijndael-192");
			break;
		case MCRYPT_RIJNDAEL_B256:
			ret = strdup("rijndael-256");
			break;
		case MCRYPT_CRYPT:
			ret = strdup("crypt");
			break;
		case MCRYPT_WAKE:
			ret = strdup("wake");
			break;
		case MCRYPT_ARCFOUR_IV:
			ret = strdup("arcfour-iv");
			break;
		case MCRYPT_RC6:
			ret = strdup("rc6");
			break;
		case MCRYPT_IDEA:
			ret = strdup("idea");
			break;
		case MCRYPT_ARCFOUR:
			ret = strdup("arcfour");
			break;
		case MCRYPT_PANAMA:
			ret = strdup("panama");
			break;	
		default:
			ret = NULL;
	}
	if (ret != NULL) {
		tolow( ret, strlen(ret));
		pointerTo_ = strchr(ret, '_');

		while (pointerTo_ != NULL) {
			*pointerTo_ = '-';
			pointerTo_ = strchr(ret, '_');
		}
	}
	return ret;	
}

char* mode_num2char(int mode) {
	char* ret, *pointerTo_;

	switch (mode) {
		case MCRYPT_ECB:
			ret = strdup("ecb");
			break;
		case MCRYPT_CBC:
			ret = strdup("cbc");
			break;
		case MCRYPT_CFB:
			ret = strdup("cfb");
			break;
		case MCRYPT_OFB:
			ret = strdup("ofb");
			break;
		case MCRYPT_nOFB:
			ret = strdup("nofb");
			break;
		case MCRYPT_STREAM:
			ret = strdup("stream");
			break;
		default:
			ret=NULL;
	}
	if (ret != NULL) {
		tolow( ret, strlen(ret));
		pointerTo_ = strchr(ret, '_');

		while (pointerTo_ != NULL) {
			*pointerTo_ = '-';
			pointerTo_ = strchr(ret, '_');
		}
	}
	return ret;	
}


int mcrypt_generic_init_compat(const int mode, const int algorithm,
			       void *key, int lenofkey, void *IV)
{
	char* _algorithm, *_mode;
	int td, tc;
	
	_algorithm = algo_num2char(algorithm);
	if (_algorithm==NULL) return MCRYPT_UNKNOWN_ERROR;
	_mode = mode_num2char(mode);
	if (_mode==NULL) {
		free(_algorithm);
		return MCRYPT_UNKNOWN_ERROR;	
	}

	td = mcrypt_module_open( _algorithm, NULL, _mode, NULL);
	free(_algorithm);
	free(_mode);

	tc = mcrypt_generic_init( td, key, lenofkey, IV);
	if (tc<0) {
		mcrypt_module_close(td);
		return tc;
	}
	
	return td;
}

int mcrypt_get_block_size_compat(int algorithm) {
char* _algorithm;
int size;

	_algorithm = algo_num2char(algorithm);
	if (_algorithm==NULL) return MCRYPT_UNKNOWN_ERROR;	
	
	size = mcrypt_get_algo_block_size( _algorithm, NULL);
	free(_algorithm);
	
	return size;

}

int mcrypt_get_key_size_compat(int algorithm) {
char* _algorithm;
int size;

	_algorithm = algo_num2char(algorithm);
	if (_algorithm==NULL) return MCRYPT_UNKNOWN_ERROR;	

	size = mcrypt_get_algo_block_size( _algorithm, NULL);
	free(_algorithm);
	
	return size;

}

int is_ok_algorithm_compat(int algorithm) {
char* _algorithm;

	_algorithm = algo_num2char(algorithm);
	if (_algorithm==NULL) return 0;

	free(_algorithm);
	return 1;
}

int is_ok_mode_compat(int mode) {
char* _mode;

	_mode = mode_num2char(mode);
	if (_mode==NULL) return 0;
	
	free(_mode);
	return 1;
}

int is_block_algorithm_compat(int algorithm) {
char* _algorithm;
int ret;

	_algorithm = algo_num2char(algorithm);
	if (_algorithm==NULL) return MCRYPT_UNKNOWN_ERROR;
	
	ret = mcrypt_is_block_algorithm(_algorithm, NULL);
	
	free(_algorithm);
	return ret;
}

int is_block_mode_compat(int mode) {
char* _mode;
int ret;

	_mode = mode_num2char(mode);
	if (_mode==NULL) return MCRYPT_UNKNOWN_ERROR;
	
	ret = mcrypt_is_block_mode(_mode, NULL);
	
	free(_mode);
	return ret;
}

char* mcrypt_get_algorithms_name_compat(int algorithm) {
char* _algorithm;

	_algorithm = algo_num2char(algorithm);
	return _algorithm;	

}
char *mcrypt_get_modes_name_compat(int mode) {
char* _mode;

	_mode = algo_num2char(mode);
	return _mode;	

}
